; PODPROGRAMY OBSUGI AKCELEROMETRU MXD2125
; Rafa Baranowski (C) 2005
;
; Etykiety procedur:
; 	Ini_akcel		- inicjalizacja akcelerometru
;	Sprawdz_akcel	- sprawdzanie przypieszenia w wybranej osi
;	Kat_akcel		- przeliczanie przypieszenia na kt wzgldem Ziemi
;	Dziel_32_bz		- obliczanie ilorazu wartoci 32-bitowych bez znaku
;
; Wymagane stae:
;	K_AKCEL		- rejestr kierunku portu akcelerometru
;	O_AKCEL		- rejestr wyjciowy portu akcelerometru
;	I_AKCEL		- rejestr wejciowy portu akcelerometru
;	AKCEL_T		- nr linii ADC dla sygnau temperatury
;	AKCEL_X		- nr linii dla sygnau przypieszenia w osi X
;	AKCEL_Y		- nr linii dla sygnau przypieszenia w osi Y


;****************************************************************
;		PROCEDURA INICJALIZUJCA AKCELEROMETR
;
Ini_akcel:
	push	R16

	in	R16, K_AKCEL
	andi	R16, ~((1<<AKCEL_T)|(1<<AKCEL_X)|(1<<AKCEL_Y))
	out	K_AKCEL, R16	; przecz linie akcelerometru w tryb wejciowy

	in	R16, O_AKCEL
	andi	R16, ~((1<<AKCEL_T)|(1<<AKCEL_X)|(1<<AKCEL_Y))
	out	O_AKCEL, R16	; wycz podciganie na liniach akcelerometru

	in	R16, TIMSK
	andi	R16, ~(1<<TOIE0)
	out	TIMSK, R16		; wycz przerwania dla licznika 0

	ldi	R16, 1<<CS00
	out	TCCR0, R16		; uruchom licznik 0, taktowany zegarem systemowym

	ldi	R16, (1<<REFS1)|AKCEL_T
	out	ADMUX, R16		; ustaw przetwornik ADC tak, aby sygnaem mierzonym
					;   by sygna temperatury; wcz wewntrzne rdo
					;   odniesienia o napiciu nominalnym 2,56 V
	ldi	R16, (1<<ADEN)|(1<<ADSC)|(1<<ADFR)|(1<<ADPS1)|(1<<ADPS0)
	out	ADCSR, R16		; uruchom przetwornik ADC w trybie cigym, przy
					;   taktowaniu z czstotliwoci 125 kHz
	pop	R16
	ret


;****************************************************************
;	PROCEDURA BADAJCA PRZYPIESZENIE W WYBRANEJ OSI
;
; Parametry:
;	R16		- maska okrelajca poddawan badaniu lini
;			  sygnau przypieszenia (X lub Y)
;
; Wartoci wyjciowe:
;	R16		- LSB wartoci przypieszenia (format [0N.8Q])
;	R17		- MSB wartoci przypieszenia (format [3N.5Q])
;	
; Uwaga: wyjciowa warto przypieszenia przekazywana jest w postaci
;	   staoprzecinkowej liczby uamkowej [3N.13Q] w kodzie U2;
;	   warto ta wyraa przypieszenie jako cz
;	   przypieszenia ziemskiego (w jednostkach g)
;
; Przeznaczenie rejestrw wewntrz procedury:
;	R18		- LSB dugoci stanu wysokiego PWM
;	R19		- MSB dugoci stanu wysokiego PWM
;	R20		- LSB dugoci okresu PWM
;	R21		- MSB dugoci okresu PWM
;
Sprawdz_akcel:
	push	R12
	push	R13
	push	R14
	push	R15
	push	R18
	push	R19
	push	R20
	push	R21
	push	R22
	push	R23

Spr_akc_czekaj_1:
	in	R17, I_AKCEL
	and	R17, R16
	brne	Spr_akc_czekaj_1	; oczekuj na stan niski sygnau PWM
	
Spr_akc_czekaj_2:
	in	R17, I_AKCEL
	and	R17, R16
	breq	Spr_akc_czekaj_2	; oczekuj na pocztek stanu wysokiego sygnau PWM

	clr	R17
	out	TCNT0, R17		; zeruj zawarto licznika 0
	ldi	R17, 1<<TOV0
	out	TIFR, R17		; i skasuj znacznik jego przepenienia

	clr	R19
Spr_akc_H_1:			; ptla wykonywana dopki sygna jest w stanie H
	in	R17, TIFR
	sbrs	R17, TOV0		; jeli nastpio przepenienie licznika 0, to...
	rjmp	Spr_akc_H_2
	inc	R19			;   inkrementuj rejestr MSB dugoci stanu H
	ldi	R17, 1<<TOV0
	out	TIFR, R17		;   i zeruj znacznik przepenienia
Spr_akc_H_2:
	in	R17, I_AKCEL	; sprawd stan badanej linii sygnaowej
	and	R17, R16
	brne	Spr_akc_H_1		; jeli jest wysoki, to zaptl
					; w przeciwnym razie...
	in	R17, TIFR
	in	R18, TCNT0		; zapamitaj zawarto licznika 0 jako LSB
					;   wartoci okrelajcej dugo stanu H

	sbrc	R17, TOV0		; jeli przed odczytem nastpio przepenienie
	rjmp	Spr_akc_popraw_H
	tst	R18
	brne	Spr_akc_ok_H	; lub jeli zawarto licznika bya zerowa
					;   (co rwnie wskazywaoby na przepenienie)
Spr_akc_popraw_H:			; to...
	inc	R19			;   inkrementuj rejestr MSB dugoci stanu H
	ldi	R17, 1<<TOV0
	out	TIFR, R17		;   i zeruj znacznik przepenienia

Spr_akc_ok_H:
	mov	R21, R19		; przepisz warto MSB dugoci stanu H
					;   do rejestru MSB dugoci okresu PWM

Spr_akc_L_1:			; ptla wykonywana dopki sygna jest stanie L
	in	R17, TIFR
	sbrs	R17, TOV0		; jeli nastpio przepenienie licznika 0, to...
	rjmp	Spr_akc_L_2
	inc	R21			;   inkrementuj rejestr MSB dugoci okresu PWM
	ldi	R17, 1<<TOV0
	out	TIFR, R17		;   i zeruj znacznik przepenienia
Spr_akc_L_2:
	in	R17, I_AKCEL	; sprawd stan badanej linii sygnaowej
	and	R17, R16
	breq	Spr_akc_L_1		; jeli jest niski, to zaptl
					; w przeciwnym razie...
	in	R17, TIFR
	in	R20, TCNT0		; zapamitaj zawarto licznika 0 jako LSB
					;   wartoci okrelajcej dugo okresu PWM

	sbrc	R17, TOV0		; jeli przed odczytem nastpio przepenienie
	rjmp	Spr_akc_popraw_L
	tst	R20
	brne	Spr_akc_ok_L	; lub jeli zawarto licznika bya zerowa
					;   (co rwnie wskazywaoby na przepenienie)
Spr_akc_popraw_L:			; to...
	inc	R21			;   inkrementuj rejestr MSB dugoci okresu PWM

Spr_akc_ok_L:
	clr	R16
	clr	R17
	clr	R22
	clr	R23
	rcall	Dziel_32_bz		; podziel pomnoon przez 2^16 dugo
					;   stanu H przez dugo okresu PWM

	subi	R16, low(0x8000)
	sbci	R17, high(0x8000)	; zamie wynik na liczb ze znakiem w kodzie U2

Spr_akc_koniec:
	pop	R23
	pop	R22
	pop	R21
	pop	R20
	pop	R19
	pop	R18
	pop	R15
	pop	R14
	pop	R13
	pop	R12
	ret


;****************************************************************
;	PROCEDURA OBLICZAJCA ILORAZ WARTOCI 32-BITOWYCH BEZ ZNAKU
;
; Parametry:
;	R16		- LSB dzielnej
;	R17		- 2. bajt dzielnej
;	R18		- 3. bajt dzielnej
;	R19		- MSB dzielnej
;	R20		- LSB dzielnika
;	R21		- 2. bajt dzielnika
;	R22		- 3. bajt dzielnika
;	R23		- MSB dzielnika
;
; Wartoci wyjciowe:
;	R12		- LSB reszty
;	R13		- 2. bajt reszty
;	R14		- 3. bajt reszty
;	R15		- MSB reszty
;	R16		- LSB ilorazu
;	R17		- 2. bajt ilorazu
;	R18		- 3. bajt ilorazu
;	R19		- MSB ilorazu
;
Dziel_32_bz:
	push	R24

	clr	R12
	clr	R13
	clr	R14
	sub	R15, R15		; zeruj reszt i znacznik C

	ldi	R24, 32+1		; zainicjalizuj licznik ptli wyk. 32+1 razy
Dz32bz_1:
	rol	R16
	rol	R17
	rol	R18
	rol	R19			; przesu dzieln/wynik w lewo

	dec	R24			; zmniejsz licznik ptli
	brne	Dz32bz_2		; jeli zakoczono, to...
	pop	R24
	ret				;   powr

Dz32bz_2:				; w przeciwnym razie...
	rol	R12
	rol	R13
	rol	R14
	rol	R15			;   wsu bit dzielnej do reszty

	sub	R12, R20
	sbc	R13, R21
	sbc	R14, R22
	sbc	R15, R23		;   zmniejsz reszt o dzielnik

	brcc	Dz32bz_3		;   jeli wynik jest ujemny, to...
	add	R12, R20
	adc	R13, R21
	adc	R14, R22		;     odtwrz reszt
	adc	R15, R23
	clc				;     i zeruj wsuwany do wyniku znacznik C
	rjmp	Dz32bz_1

Dz32bz_3:				;   w przeciwnym razie...
	sec				;     ustaw wsuwany do wyniku znacznik C
	rjmp	Dz32bz_1


;****************************************************************
;	PROCEDURA PRZELICZAJCA PRZYPIESZENIE NA KT WZGLDEM ZIEMI
;
; Parametry:
;	R16		- LSB wartoci przypieszenia (format [0N.8Q])
;	R17		- MSB wartoci przypieszenia (format [3N.5Q])
;
; Wartoci wyjciowe:
;	R17		- kt wzgldem Ziemi
;	T (SREG)	- znak kta (T=1 => kt ujemny)
;
Kat_akcel:
	push	R18
	push	ZL
	push	ZH

	bst	R17, 7		; zapisz znak przypieszenia w T w SREG
	brtc	Kat_akc_dod		; jeli przypieszenie jest ujemne, to...

	com	R16
	com	R17
	subi	R16, low(-1)
	sbci	R17, high(-1)	;   zaneguj jego warto w kodzie U2

Kat_akc_dod:
	mov	R18, R17
	andi	R18, 0xE0
	brne	Kat_akc_ovf		; jeli przypieszenie jest mniejsze ni 1g

	lsl	R16
	rol	R17
	lsl	R16
	rol	R17
	lsl	R16			;   przesu cz uamkow do pocztku MSB,
	rol	R17			;     tak aby utworzy liczb formatu [0N.16Q]

	rjmp	Kat_akc_koniec
	
Kat_akc_ovf:			; w przeciwnym razie...
	ser	R17			;   ustaw warto maksymaln
	rjmp	Kat_akc_koniec

Kat_akc_koniec:
	clr	R16
	ldi	ZL, low(Konwersja_akcel<<1)
	ldi	ZH, high(Konwersja_akcel<<1)
	add	ZL, R17
	adc	ZH, R16

	lpm	R17, Z		; przelicz przypieszenie na kt poprzez
					;   tablic konwersji "Konwersja_akcel"
	pop	ZH
	pop	ZL
	pop	R18
	ret

Konwersja_akcel:			; tablica konwersji liczby uamkowej [0N.8Q]
.DB 0, 0, 0, 1, 1, 1, 1, 2	;   na warto arcus sinus (deg)
.DB 2, 2, 2, 2, 3, 3, 3, 3
.DB 4, 4, 4, 4, 4, 5, 5, 5
.DB 5, 6, 6, 6, 6, 7, 7, 7
.DB 7, 7, 8, 8, 8, 8, 9, 9
.DB 9, 9, 9, 10, 10, 10, 10, 11
.DB 11, 11, 11, 12, 12, 12, 12, 12
.DB 13, 13, 13, 13, 14, 14, 14, 14
.DB 15, 15, 15, 15, 15, 16, 16, 16
.DB 16, 17, 17, 17, 17, 18, 18, 18
.DB 18, 19, 19, 19, 19, 19, 20, 20
.DB 20, 20, 21, 21, 21, 21, 22, 22
.DB 22, 22, 23, 23, 23, 23, 24, 24
.DB 24, 24, 25, 25, 25, 25, 26, 26
.DB 26, 26, 27, 27, 27, 27, 28, 28
.DB 28, 28, 29, 29, 29, 29, 30, 30
.DB 30, 30, 31, 31, 31, 31, 32, 32
.DB 32, 32, 33, 33, 33, 34, 34, 34
.DB 34, 35, 35, 35, 35, 36, 36, 36
.DB 37, 37, 37, 37, 38, 38, 38, 39
.DB 39, 39, 39, 40, 40, 40, 41, 41
.DB 41, 42, 42, 42, 42, 43, 43, 43
.DB 44, 44, 44, 45, 45, 45, 46, 46
.DB 46, 47, 47, 47, 47, 48, 48, 49
.DB 49, 49, 50, 50, 50, 51, 51, 51
.DB 52, 52, 52, 53, 53, 54, 54, 54
.DB 55, 55, 55, 56, 56, 57, 57, 57
.DB 58, 58, 59, 59, 60, 60, 61, 61
.DB 61, 62, 62, 63, 63, 64, 64, 65
.DB 65, 66, 67, 67, 68, 68, 69, 70
.DB 70, 71, 72, 72, 73, 74, 75, 76
.DB 77, 78, 79, 80, 81, 83, 85, 90
