; Najprostszy oscyloskop cyfrowy, obsugiwany mysz PS/2
; Rafa Baranowski (C) 2005

.INCLUDE "tn26def.inc"		; SEGMENT DEKLARACJI

.EQU SYS_FREQ	= 1		; czstotliwo systemowa w MHz
					; stae oscyloskopu
.EQU OSC_TRIG	= 0		;   nr linii ADC dla sygnau poziomu wyzwalania
.EQU OSC_SYG	= 5		;   nr linii ADC dla sygnau badanego (AIN0/ADC5)
					; stae interfejsu PS/2 i myszy
.EQU ROZDZ_MYSZY	= 1		;   rozdzielczo myszy (0...3, 0 - minimum)
.EQU K_PS2_CLK	= DDRA	;   rejestr kierunku dla linii CLK
.EQU O_PS2_CLK	= PORTA	;   rejestr wyjciowy dla linii CLK
.EQU I_PS2_CLK	= PINA	;   rejestr wejciowy dla linii CLK
.EQU PS2_CLK	= 7		;   nr linii dla sygnau CLK
.EQU K_PS2_DTA	= DDRB	;   rejestr kierunku dla linii DTA
.EQU O_PS2_DTA	= PORTB	;   rejestr wyjciowy dla linii DTA
.EQU I_PS2_DTA	= PINB	;   rejestr wejciowy dla linii DTA
.EQU PS2_DTA	= 6		;   nr linii dla sygnau DTA
					; stae wywietlacza LCD
.EQU K_LCD		= DDRB	;   rejestr kierunku portu
.EQU O_LCD		= PORTB	;   rejestr wyjciowy portu
.EQU LCD_DB4	= 0		;   nr linii dla sygnau DB4
.EQU LCD_DB5	= 1		;   nr linii dla sygnau DB5
.EQU LCD_DB6	= 2		;   nr linii dla sygnau DB6
.EQU LCD_DB7	= 3		;   nr linii dla sygnau DB7
.EQU LCD_RS		= 4		;   nr linii dla sygnau RS
.EQU LCD_EN		= 5		;   nr linii dla sygnau EN

					; nazwy symboliczne rejestrw roboczych
.DEF USTAW_YM	= R13		;   rejestr wzmocnienia Y (1/2/4/8/16/32)
.DEF USTAW_YP	= R12		;   rejestr przesunicia Y (-128...127)
.DEF USTAW_SP	= R14		;   rejestr okresu prbkowania (1...7)
.DEF KURSOR_LCD	= R15		;   rejestr pooenia kursora LCD


.DSEG					; SEGMENT DANYCH
.ORG	0x0060			; pocztek pamici SRAM
Bufor_syg:		.BYTE 40	;   bufor 40 prbek sygnau badanego
Znak_LCD:		.BYTE 8	;   bufor szablonu znaku LCD
Mysz_X:		.BYTE 1	;   zmienna pooenia myszy w poziomie


.CSEG					; SEGMENT PROGRAMU
.ORG	0				; wektor zerowania
	rjmp	Reset			;   skok do procedury inicjalizujcej
.ORG	OVF0addr			; wektor przerwania TOV0
	rjmp	Brak_wyzw		;   skok do procedury obsugi braku wyzwalania
.ORG	OVF1addr			; wektor przerwania TOV1
	rjmp	Blad_PS2		;   skok do procedury obsugi bdu PS/2
.ORG	ACIaddr			; wektor przerwania dla komparatora analogowego
	rjmp	Wyzwalanie		;   skok do procedury wyzwalajcej prbkowanie

					; deklaracje staych w pamici programu
T_powitanie:	.db	"   Oscyloskop   ", ZNAK_KONCA
T_menu:		.db	"YM:   YP:   SP: ", ZNAK_KONCA 
T_blad:		.db	"   Blad myszy!", ZNAK_KONCA

Reset:				; podprogram inicjalizujcy
	ldi	R16, RAMEND
	out	SP, R16		; inicjalizacja wskanika stosu

	ldi	R16, 1<<SE
	out	MCUCR, R16		; zezwl na tryb upienia w wariancie Idle

	ldi	R16, (1<<CS02)|(1<<CS00)
	out	TCCR0, R16		; uruchom licznik 0, taktowany zegarem systemowym,
					;   podzielonym w stosunku 1:1024 (przepenienia
					;   co okoo 0,26 s przy zegarze 1 MHz)
	clr	R16
	sts	Mysz_X, R16		; zeruj zmienn pooenia myszy w poziomie
	ldi	R16, 0x40
	mov	KURSOR_LCD, R16	; ustal pooenie kursora LCD w linii dolnej
	ldi	R16, 1
	mov	USTAW_YM, R16	; ustal wzmocnienie Y rwne 1
	clr	USTAW_YP		; ustal przesunicie Y rwne 0
	mov	USTAW_SP, R16	; ustal okres prbkowania na minimalny
					;   (czstotliwo prbkowania rwna 38,5 kHz)

	rcall	Ini_LCD		; inicjalizacja wywietlacza LCD
	ldi	ZH, high(T_powitanie<<1)
	ldi	ZL, low(T_powitanie<<1)
	rcall	Pokaz_ciag_LCD	; wywietlanie powitania
	ldi	R16, 100
	ldi	R17, 10
	rcall	Czekaj_ms		; opnienie o 1 s
	rcall	Czysc_LCD		; czyszczenie wywietlacza

	ldi	R16, 4|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przesu kursor LCD na 4 pozycj grnej linii
	clr	R17
Ptl_wypelnij_LCD:			; adowanie kolejnych pozycji pamici DDRAM
	mov	R16, R17		;   wywietlacza LCD kolejnymi kodami znakw
	rcall	Zapisz_LCD		;   specjalnych; na pozycjach tych wizualizowane
	inc	R17			;   bd fragmenty badanego przebiegu
	cpi	R17, 8
	brlo	Ptl_wypelnij_LCD

	sei				; wcz obsug przerwa

	rcall	Ini_myszy_PS2	; inicjalizacja myszy PC
	brtc	Petla			; jeli przebiega pomylnie (T=0),
					;   to przejd do wykonywania ptli gwnej
					; jeli jednak wystpi bd (T=1), to...
	rcall	Czysc_LCD		;   wyczy wywietlacz
	ldi	ZH, high(T_blad<<1)
	ldi	ZL, low(T_blad<<1)
	rcall	Pokaz_ciag_LCD	;   wywietl komunikat bdu
	rjmp	PC			;   i zawie dalsze wykonywanie programu

Petla:				; ptla gwna programu
	ldi	R16, 0x40|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD
	ldi	ZH, high(T_menu<<1)
	ldi	ZL, low(T_menu<<1)
	rcall	Pokaz_ciag_LCD	; wywietlanie menu ustawie oscyloskopu
	ldi	R16, (0x40+3)|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD
	mov	R16, USTAW_YM
	rcall	Pokaz_bajt_LCD	; wywietlanie aktualnego wzmocnienia Y
	ldi	R16, (0x40+9)|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD
	mov	R16, USTAW_YP
	rcall	Pokaz_bajt_U2_LCD	; wywietlanie aktualnego przesunicia Y
	ldi	R16, (0x40+15)|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD
	mov	R16, USTAW_SP
	rcall	Pokaz_bajt_LCD	; wywietlanie aktualnego okresu prbkowania
	ldi	R16, 1<<LCD_DDADR
	or	R16, KURSOR_LCD
	rcall	Instrukcja_LCD	; przesuwanie kursora w miejsce wyznaczone mysz

	ldi	R16, 100
	ldi	R17, 1
	rcall	Czekaj_ms		; min. okres odwieania wywietlacza wynosi 100ms

	ldi	R16, 15|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD
	ldi	R16, ' '		; czyszczenie pozycji znaku informujcego o braku
	rcall	Zapisz_LCD		;   wyzwalania (ostatniej pozycji grnej linii LCD)
	ldi	R16, 1<<LCD_DDADR
	or	R16, KURSOR_LCD
	rcall	Instrukcja_LCD	; przesuwanie kursora w miejsce wyznaczone mysz

	ldi	R16, OSC_TRIG
	out	ADMUX, R16		; przecz multiplekser ADC na sygna poziomu wyzw.
	ldi	R16, (1<<ACME)|(1<<ACI)|(1<<ACIS1)|(1<<ACIS0)
	out	ACSR, R16		; wcz komparator analogowy; sygnaem doczonym
					;   do wejcia "+" jest sygna badany, sygnaem
					;   doczonym do wejcia "-" jest sygna poziomu
					;   wyzwalania (doczony poprzez multiplexer ADC)

	clr	R16
	out	TCNT0, R16		; zeruj zawarto licznika 0
	ldi	R16, 1<<TOV0
	out	TIFR, R16		; anuluj ewentualne danie przerwania TOV0
	in	R16, TIMSK
	ori	R16, 1<<TOIE0
	out	TIMSK, R16		; zezwl na przerwania TOV0

	ldi	R16, (1<<ACME)|(1<<ACIE)|(1<<ACI)|(1<<ACIS1)|(1<<ACIS0)
	out	ACSR, R16		; anuluj ewentualne danie przerwania ACI i
					;   wcz obsug przerwa dla moduu komparatora
					;   (przy zboczu narastajcym jego sygn. wyj.)

	sleep				; przejd w tryb upienia (Idle)
					;   mikrokontroler wychodzi z tego trybu
					;   wskutek wywoania przerwania dla komparatora
					;   analogowego (wyzwolenie sygnaem) lub
					;   wskutek przepenienia licznika 0 (wyzwolenie
					;   sztuczne)

	rcall Sprawdz_mysz_PS2	; sprawd stan myszy; teraz:
					;   R18 - zawiera stan przyciskw
					;   R17 - zawiera przemieszczenie X
					;   R16 - zawiera przemieszczenie Y

	tst	R18			; jeli aden klawisz nie zosta nacinity,
	breq	Ptl_przemiesc	;   to przejd do obsugi przemieszczenia

	mov	R17, KURSOR_LCD
Ptl_klaw_L:
	sbrs	R18, M_KL_L		; jeli nacinito lewy klawisz myszy, to...
	rjmp	Ptl_klaw_P
	cpi	R17, 0x40+6
	brsh	Ptl_klaw_L_1	;   jeli kursor LCD znajdowa si na pozycji YM:
	lsl	USTAW_YM		;     zwiksz warto wzmocnienia Y dwukrotnie
	lsl	USTAW_YP		;     zwiksz warto przesunicia Y dwukrotnie
	mov	R16, USTAW_YM	;     i dopilnuj, aby wzmocnienie Y wynosio 1, 2,
	cpi	R16, 1<<6		;       4, 8, 16 lub 32
	brlo	Ptl_klaw_kon
	lsr	USTAW_YM
	lsr	USTAW_YP
	rjmp	Ptl_klaw_kon
Ptl_klaw_L_1:
	cpi	R17, 0x40+12
	brsh	Ptl_klaw_L_2	;   jeli kursor LCD znajdowa si na pozycji YP:
	inc	USTAW_YP		;     inkrementuj warto przesunicia Y
	rjmp	Ptl_klaw_kon
Ptl_klaw_L_2:			;   jeli kursor LCD znajdowa si na pozycji SP:
	inc	USTAW_SP		;     inkrementuj warto okresu prbkowania
	sbrs	USTAW_SP, 3		;     i dopilnuj, by zawierao si ono
	rjmp	Ptl_klaw_kon	;       w przedziale 1...7
	dec	USTAW_SP
	rjmp	Ptl_klaw_kon

Ptl_klaw_P:
	sbrs	R18, M_KL_P		; jeli nacinito prawy klawisz myszy, to...
	rjmp	Ptl_klaw_kon
	cpi	R17, 0x40+6
	brsh	Ptl_klaw_P_1	;   jeli kursor LCD znajdowa si na pozycji YM:
	lsr	USTAW_YM		;     zmniejsz warto wzmocnienia Y dwukrotnie
	asr	USTAW_YP		;     zmniejsz warto przesunicia Y dwukrotnie
	tst	USTAW_YM		;     i dopilnuj, aby wzmocnienie Y wynosio 1, 2,
	brne	Ptl_klaw_kon	;       4, 8, 16 lub 32
	sec
	rol	USTAW_YM
	lsl	USTAW_YP
	rjmp	Ptl_klaw_kon
Ptl_klaw_P_1:
	cpi	R17, 0x40+12
	brsh	Ptl_klaw_P_2	;   jeli kursor LCD znajdowa si na pozycji YP:
	dec	USTAW_YP		;     dekrementuj warto przesunicia Y
	rjmp	Ptl_klaw_kon
Ptl_klaw_P_2:			;   jeli kursor LCD znajdowa si na pozycji SP:
	dec	USTAW_SP		;     dekrementuj warto okresu prbkowania
	brne	Ptl_klaw_kon	;     i dopilnuj, by zawierao si ono
	inc	USTAW_SP		;       w przedziale 1...7


Ptl_klaw_kon:
	rcall Sprawdz_mysz_PS2	; ponownie sprawd stan myszy
	andi	R18, (1<<M_KL_L)|(1<<M_KL_P)
	tst	R18
	brne	Ptl_klaw_kon	; i oczekuj na zwolnienie klawiszy
	rjmp	Petla			; a nastpnie powr do wykonywania ptli gwnej


Ptl_przemiesc:			; obsuga przemieszczenia myszy:
	lds	R20, Mysz_X		;   odczytaj poprzedni pozycj myszy w osi X
	mov	R21, R20
	add	R21, R17		;   uwzgldnij stwierdzone przemieszczenie X
	sts	Mysz_X, R21		;   i zapamitaj aktualn pozycj myszy w osi X

	eor	R20, R21		;   porwnaj poprzedni i aktualn pozycj X
	andi	R20, 0xF0		;   zamaskuj modsz tetrad porwnania
	brne	Ptl_przem_1		;   jeli starsze tetrady nie zmieniy si,
	rjmp	Petla			;     to przejd do wykonywania ptli gwnej
					;   w przeciwnym razie...
Ptl_przem_1:
	swap	R21
	andi	R21, 0x0F		;     wyuskaj przemieszczenie kursora w X
	ori	R21, 0x40		;	zamie przemieszczenie na pozycj na LCD
	mov	KURSOR_LCD, R21	;     i zapamitaj wynik jako now pozycj kursora

	rjmp	Petla			; zaptl program


;*******************************************************************************
Brak_wyzw:				; PROCEDURA OBSUGI PRZERWANIA TOV0
	push	R16			;   (sztuczne wyzwalanie prbkowania)
	in	R16, SREG
	push	R16
		
	ldi	R16, 15|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; na kocu grnej linii LCD wywietl liter "A",
	ldi	R16, 'A'		;   sygnalizujc brak poprawnego wyzwalania i
	rcall	Zapisz_LCD		;   uaktywnienie wyzwalania automatycznego
	ldi	R16, 1<<LCD_DDADR
	or	R16, KURSOR_LCD
	rcall	Instrukcja_LCD	; przesuwanie kursora w miejsce wyznaczone mysz
		
	rcall	Wyzwalanie		; sztuczne wyzwalanie prbkowania

	pop	R16
	out	SREG, R16
	pop	R16
	reti


;*******************************************************************************
Wyzwalanie:				; PROCEDURA OBSUGI PRZERWANIA ACI
	push	R16			;   (wyzwalanie prbkowania)
	push	R17
	push	R18
	push	R19
	push	XL
	push	XH
	in	R16, SREG
	push	R16

	in	R16, TIMSK
	andi	R16, ~(1<<TOIE0)
	out	TIMSK, R16		; zablokuj obsug przerwa TOV0
	ldi	R16, 1<<ACD
	out	ACSR, R16		; wycz komparator analogowy i jego przerwania

	ldi	R16, (1<<ADLAR)|OSC_SYG
	out	ADMUX, R16		; przecz multiplekser ADC na sygna badany;
					;   rdem odniesienia niech jest napicie
					;   AVCC (5 V); wcz opcj lewostronnego
					;   przedstawiania wyniku konwersji
	mov	R16, USTAW_SP
	ori	R16, (1<<ADEN)|(1<<ADFR)|(1<<ADSC)|(1<<ADIF)
	out	ADCSR, R16		; wcz przetwornik ADC, pracujcy w trybie
					;   cigym, taktowany zegarem systemowym,
					;   podzielonym w stosunku 1:2^(USTAW_SP)

	ldi	XL, low(Bufor_syg)
	ldi	XH, high(Bufor_syg)
	ldi	R17, 40
Wyzw_probkuj:			; wypenianie zmiennej tablicowej "Bufor_syg"
	in	R16, ADCSR		;   w pamici SRAM czterdziestoma kolejnymi
	sbrs	R16, ADIF		;   prbkami sygnau badanego
	rjmp	Wyzw_probkuj
	out	ADCSR, R16
	in	R16, ADCH
	st	X+, R16
	dec	R17
	brne	Wyzw_probkuj

	clr	R16
	out	ADCSR, R16		; wycz przetwornik ADC

	ldi	R16, 0|(1<<LCD_CGADR)
	rcall	Instrukcja_LCD	; wejcie w tryb adresowania pamici CGRAM LCD
					;   (proc. "Zapis_LCD" bdzie zapisywaa dane
					;   do pamici generatora znakw - CGRAM)

	ldi	R18, 1<<4		; inicjalizuj wskanik kolumny punktu w znaku
	clr	R19			; inicjalizuj licznik prbek sygnau badanego
Wyzw_wyswietlaj:			; ptla przetwarzajca 40 prbek sygnau z
	clr	R16			;   tablicy "Bufor_syg" na 8 szablonw LCD
	ldi	XL, low(Bufor_syg)
	ldi	XH, high(Bufor_syg)
	add	XL, R19		;   zaaduj wskanik X adresem aktualnie
	adc	XH, R16		;     przetwarzanej prbki sygnau ze SRAM

	ld	R17, X		;   zaaduj kolejn prbk do rejestru R17

	push	USTAW_YM
	lsl	USTAW_YM
	lsl	USTAW_YM
	lsl	USTAW_YM
	tst	USTAW_YM
Wyzw_przesuwaj:			;   podziel warto prbki przez liczb 2,
	breq	Wyzw_1		;     podniesion do potgi zalenej od aktualnej
	lsr	R17			;     wartoci wzmocnienia Y (zawartoci rejestru
	lsl	USTAW_YM		;     USTAW_YM)
	rjmp	Wyzw_przesuwaj
Wyzw_1:
	pop	USTAW_YM

	add	R17, USTAW_YP	;   do wartoci prbki dodaj aktualne przesunicie Y
					;     (zawarto rejestru USTAW_YP)
	cpi	R17, 8
	brsh	Wyzw_poza		;   jeli warto prbki nie ley poza zakresem, to...

	ldi	R16, 7
	sub	R16, R17		;     oblicz uzupenienie wartoci prbki do siedmiu

	clr	R17
	ldi	XL, low(Znak_LCD)
	ldi	XH, high(Znak_LCD)
	add	XL, R16		;     zaaduj wskanik X adresem wiersza szablonu
	adc	XH, R17		;       znakowego, zalenym od wartoci prbki

	ld	R17, X
	or	R17, R18		;     w kolejnej kolumnie (R18) zaadresowanego
	st	X, R17		;       wiersza szablonu znakowego uaktywnij punkt,
					;       symbolizujcy aktualnie przetwarzan
					;       prbk sygnau badanego
Wyzw_poza:
	lsr	R18			;   przesu wskanik kolumny punktu w znaku w prawo
	brcc	Wyzw_2		;   jeli by to ostatni punkt znaku, to...
	ldi	R18, 1<<4

	ldi	XL, low(Znak_LCD)
	ldi	XH, high(Znak_LCD)
	ldi	R17, 8
Wyzw_zap_CGRAM:			;     zapisz kolejny szablon znaku w pam. CGRAM LCD
	ld	R16, X+
	rcall	Zapisz_LCD
	dec	R17
	brne	Wyzw_zap_CGRAM

	ldi	R16, 8
	clr	R17
	ldi	XL, low(Znak_LCD)
	ldi	XH, high(Znak_LCD)
Wyzw_czysc_znak_2:		;     wyczy zmienn tablicow "Znak_LCD"
	st	X+, R17		;       w pamici SRAM; w zmiennej tej tworzony
	dec	R16			;       bdzie kolejny szablon znaku LCD dla
	brne	Wyzw_czysc_znak_2	;       kolejnych piciu prbek sygnau badanego

Wyzw_2:
	inc	R19
	cpi	R19, 40
	brlo	Wyzw_wyswietlaj	; zaptl iteracyjnie

	pop	R16
	out	SREG, R16
	pop	XH
	pop	XL
	pop	R19
	pop	R18
	pop	R17
	pop	R16
	reti


					; DOCZANE PODPROGRAMY
.INCLUDE "czekaj_ms.inc"	;   oczekiwanie milisekundowe
.INCLUDE "czekaj_us.inc"	;   oczekiwanie mikrosekundowe
.INCLUDE "mysz_ps2.inc"		;   obsuga myszy PS/2
.INCLUDE "lcd.inc"		;   obsuga wywietlacza LCD
