; Kostka do gry, obsuga moduu LCD (HD44780)
; Rafa Baranowski (C) 2005

.INCLUDE "tn26def.inc"		; SEGMENT DEKLARACJI

.EQU	KOSTKA	= 10		; liczba "k" kostki (1...255)
.EQU	SYS_FREQ	= 1		; czstotliwo pracy w MHz
					; stae klawiatury
.EQU	K_KLAW	= DDRB	;   rejestr kierunku portu klawiatury
.EQU	O_KLAW	= PORTB	;   rejestr wyjciowy portu klawiatury
.EQU	I_KLAW	= PINB	;   rejestr wejciowy portu klawiatury
.EQU	KL_UNIV1	= 4		;   nr linii dla klawisza UNIV1
					; stae wywietlacza LCD
.EQU K_LCD		= DDRB	;   rejestr kierunku portu wywietlacza
.EQU O_LCD		= PORTB	;   rejestr wyjciowy portu wywietlacza
.EQU LCD_DB4	= 0		;   nr linii dla sygnau DB4
.EQU LCD_DB5	= 1		;   nr linii dla sygnau DB5
.EQU LCD_DB6	= 2		;   nr linii dla sygnau DB6
.EQU LCD_DB7	= 3		;   nr linii dla sygnau DB7
.EQU LCD_RS		= 4		;   nr linii dla sygnau RS
.EQU LCD_EN		= 5		;   nr linii dla sygnau EN

.CSEG					; SEGMENT PROGRAMU
.ORG	0				; wektor zerowania
	rjmp	Reset			; skok do procedury inicjalizujcej

.ORG	PCI0addr			; wektor przerwania PCI0
	reti				; pusta procedura obsugi (powrt)

					; deklaracje staych w pamici programu
T_powitanie:	.db	"Kostka do gry", ZNAK_KONCA
T_liczba_k:		.db	"Liczba k: ", ZNAK_KONCA
T_zacheta:		.db	"Rzu", POL_C, " kostk", POL_A, ZNAK_KONCA
T_wynik:		.db	"Wynik: ", ZNAK_KONCA

Reset:				; podprogram inicjalizujcy
	ldi	R16, RAMEND
	out	SP, R16		; inicjalizacja wskanika stosu

	ldi	R16, 0xFF
	out	PORTA, R16		; wcz podciganie na wszystkich portach
	out	PORTB, R16		;   mikrokontrolera (konieczne, poniewa
					;   przerwanie PCI bdzie wykorzystywane do
					;   wyprowadzania ukadu z trybu upienia

	rcall	Ini_LCD		; inicjalizacja wywietlacza LCD

	ldi	R16, (1<<LCD_STER)|(1<<LCD_SW_ON)|(1<<LCD_SK_OFF)|(1<<LCD_SM_OFF)
	rcall	Instrukcja_LCD	; wyczanie kursora

	ldi	ZH, high(Polskie_znaki<<1)
	ldi	ZL, low(Polskie_znaki<<1)
	rcall	Zapisz_CGRAM	; zapisywanie pamici CGRAM ksztatami polskich
					;   znakw diakrytycznych

	ldi	ZH, high(T_powitanie<<1)
	ldi	ZL, low(T_powitanie<<1)
	rcall	Pokaz_ciag_LCD	; wywietlenie powitania "Kostka do gry" na
					;   pocztku pierwszej linii wywietlacza

	ldi	R16, 0x40|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przechodzenie do drugiej linii wywietlacza

	ldi	ZH, high(T_liczba_k<<1)
	ldi	ZL, low(T_liczba_k<<1)
	rcall	Pokaz_ciag_LCD	; wywietlanie etykiety "Liczba k: " na
					;   pocztku drugiej linii wywietlacza	

	ldi	R16, KOSTKA
	rcall	Pokaz_bajt_LCD	; wywietlanie liczby "k" kostki

	ldi	R17, 100
	ldi	R16, 10
	rcall	Czekaj_ms		; opnienie o 1 s

	rcall	Czysc_LCD		; czyszczenie wywietlacza

	ldi	ZH, high(T_zacheta<<1)
	ldi	ZL, low(T_zacheta<<1)
	rcall	Pokaz_ciag_LCD	; wywietlanie zachty "Rzu kostk" na
					;   pocztku pierwszej linii wywietlacza	

	ldi	R16, 0x40|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przechodzenie do drugiej linii wywietlacza

	ldi	ZH, high(T_wynik<<1)
	ldi	ZL, low(T_wynik<<1)
	rcall	Pokaz_ciag_LCD	; wywietlanie etykiety "Wynik: " na
					;   pocztku drugiej linii wywietlacza

	ldi	R16, (1<<LCD_STER)|(1<<LCD_SW_ON)|(1<<LCD_SK_ON)|(1<<LCD_SM_ON)
	rcall	Instrukcja_LCD	; wczanie migajcego kursora


	ldi	R16, (1<<SE)|(1<<SM1)
	out	MCUCR, R16		; odblokuj moliwo wczania trybu
					;   upienia w wariancie PowerDown

	ldi	R16, (1<<PCIE1)
	out	GIMSK, R16		; docz m.in. lini klawisza UNIV1 (PB4)
					;   do moduu przerwa zewntrznych PCI i
					;   zezwl na pochodzce od niego przerwania

	ldi	R16, (1<<CTC1)|(1<<CS00)
	out	TCCR1B, R16		; uruchom licznik 1 w trybie CTC, taktowany
					;   zegarem systemowym (1 MHz)

	ldi	R16, KOSTKA-1
	out	OCR1C, R16		; ustal pojemno licznika 1 rwn
					;   zakresowi losowania


Petla:				; ptla gwna programu
	cbi	K_KLAW, KL_UNIV1	; przecz lini klawisza UNIV1 w tryb wej.

	ldi	R16, (1<<PCIF)
	out	GIFR, R16		; anuluj ewentualne danie przerwania PCI

	sei				; odblokuj obsug przerwa

	sleep				; przejd w tryb upienia
					;  dalsza cz programu wykonywana jest po 
					;  zmianie stanu klawisza UNIV1
	
	cli				; chwilowo zablokuj obsug przerwa

	ldi	R16, 10
	ldi	R17, 1
	rcall	Czekaj_ms		; oczekiwanie przez 10 ms, pozwalajce uchroni
					;   si przed bdem wynikajcym z drga stykw

	sbic	I_KLAW, KL_UNIV1	; sprawd stan klawisza UNIV1
	rjmp	Petla			; jeli jest puszczony, wr do trybu upienia

Losuj:
	sbis	I_KLAW, KL_UNIV1	; jeli klawisz UNIV1 jest przetrzymywany...
	rjmp	Losuj			;   to czekaj na jego puszczenie

	ldi	R16, 10
	ldi	R17, 1
	rcall	Czekaj_ms		; oczekiwanie przez 10 ms, pozwalajce uchroni
					;   si przed bdem wynikajcym z drga stykw

	sbis	I_KLAW, KL_UNIV1	; sprawd ponownie, czy klawisz UNIV1 rzeczywicie
	rjmp	Losuj			;   zosta puszczony

	sbi	K_KLAW, KL_UNIV1	; przecz lini klawisza UNIV1 w tryb wyj.
					;   (to niezbdne, poniewa jest ona jednoczenie
					;   przyporzdkowana linii RS wywietlacza LCD)

	ldi	R16, (0x40+7)|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przesu kursor na miejsce, w ktrym wywietlany
					;   bdzie wynik (7 pozycja drugiej linii)

	in	R16, TCNT1
	inc	R16
	rcall	Pokaz_bajt_LCD	;   wywietl wylosowan warto

	ldi	R16, 2
	rcall	Spacja_LCD		;   wywietl dwa puste znaki (mazanie ewentualnej
					;     pozostaoci poprzedniego wyniku)

	rjmp	Petla			; zaptl program


					; DOCZANE PODPROGRAMY
.INCLUDE "czekaj_ms.inc"	;   oczekiwanie milisekundowe
.INCLUDE "czekaj_us.inc"	;   oczekiwanie mikrosekundowe
.INCLUDE "lcd.inc"		;   obsuga wywietlacza LCD
