; Zegarek na RTC DS1307, obsuga interfejsu I2C poprzez modu USI
; Rafa Baranowski (C) 2005

.INCLUDE "tn26def.inc"		; SEGMENT DEKLARACJI

.EQU	SYS_FREQ	= 1		; czstotliwo pracy w MHz
.EQU	RTC_ADR	= 0xD0	; adres ukadu RTC DS1307 na magistrali I2C
					; stae interfejsu I2C
.EQU	K_USI		= DDRB	;   rejestr kierunku portu USI
.EQU	O_USI		= PORTB	;   rejestr wyjciowy portu USI
.EQU	I_USI		= PINB	;   rejestr wejciowy portu USI
.EQU	USI_SDA	= 0		;   nr linii dla sygnau DI/SDA
.EQU	USI_SCL	= 2		;   nr linii dla sygnau USCK/SCL

.EQU	K_I2C_OFF	= DDRA	;   rejestr kierunku portu dla linii I2C_OFF
.EQU	O_I2C_OFF	= PORTA	;   rejestr wyjciowy portu dla linii I2C_OFF
.EQU	I_I2C_OFF	= PINA	;   rejestr wejciowy portu dla linii I2C_OFF
.EQU	I2C_OFF	= 2		;   nr linii dla sygnau I2C_OFF
					; stae klawiatury
.EQU	K_KLAW	= DDRA	;   rejestr kierunku portu klawiatury 
.EQU	O_KLAW	= PORTA	;   rejestr wyjciowy portu klawiatury
.EQU	I_KLAW	= PINA	;   rejestr wejciowy portu klawiatury
.EQU	KL_A		= 0		;   nr linii dla klawisza A
.EQU	KL_B		= 1		;   nr linii dla klawisza B
					; stae wywietlacza LCD
.EQU	K_LCD		= DDRB	;   rejestr kierunku portu wywietlacza
.EQU	O_LCD		= PORTB	;   rejestr wyjciowy portu wywietlacza
.EQU	LCD_DB4	= 0		;   nr linii dla sygnau DB4
.EQU	LCD_DB5	= 1		;   nr linii dla sygnau DB5
.EQU	LCD_DB6	= 2		;   nr linii dla sygnau DB6
.EQU	LCD_DB7	= 3		;   nr linii dla sygnau DB7
.EQU	LCD_RS	= 4		;   nr linii dla sygnau RS
.EQU	LCD_EN	= 5		;   nr linii dla sygnau EN


.DSEG					; SEGMENT DANYCH
.ORG 0x0060				; pocztek pamici SRAM
RTC_Data:		.BYTE	7	; zmienna stanu RTC

.CSEG					; SEGMENT PROGRAMU
.ORG	0				; wektor zerowania
	rjmp	Reset			; skok do procedury inicjalizujcej


					; deklaracje staych w pamici programu
T_powitanie:	.db	"Zegarek - DS1307", ZNAK_KONCA
T_szablon1:		.db	"Dz. X   GG:MM:SS", ZNAK_KONCA
T_szablon2:		.db	"    DD.MM.RR r. ", ZNAK_KONCA
T_blad:		.db	"B", POL_L, POL_A, "d uk", POL_L, "adu RTC!", ZNAK_KONCA


Reset:				; podprogram inicjalizujcy
	ldi	R16, RAMEND
	out	SP, R16		; inicjalizacja wskanika stosu

	sbi	O_KLAW, KL_A
	sbi	O_KLAW, KL_B	; wcz podciganie na liniach klawiatury

	sbi	K_I2C_OFF, I2C_OFF; przecz lini I2C_OFF w tryb wyjciowy
	sbi	O_I2C_OFF, I2C_OFF; zablokuj magistral I2C

	rcall	Ini_LCD		; inicjalizacja wywietlacza LCD	
	ldi	R16, (1<<LCD_STER)|(1<<LCD_SW_ON)|(1<<LCD_SK_OFF)|(1<<LCD_SM_OFF)
	rcall	Instrukcja_LCD	; wyczanie kursora LCD

	ldi	ZH, high(Polskie_znaki<<1)
	ldi	ZL, low(Polskie_znaki<<1)
	rcall	Zapisz_CGRAM	; zapisywanie pamici CGRAM ksztatami
					;   polskich znakw diakrytycznych

	ldi	ZH, high(T_powitanie<<1)
	ldi	ZL, low(T_powitanie<<1)
	rcall	Pokaz_ciag_LCD	; wywietlanie powitania

	ldi	R17, 100
	ldi	R16, 10
	rcall	Czekaj_ms		; opnienie o 1 s
	rcall	Czysc_LCD		; czyszczenie wywietlacza

	rcall	Ini_USI_I2C		; inicjalizacja moduu USI w nadrzdnym trybie I2C
	rcall	Czytaj_RTC		; przepisywanie zawartoci pamici RTC do
					;   zm. tablicowej RTC_Data w pamici SRAM
	ldi	YL, low(RTC_Data)	; adowanie wskanika Y adresem
	ldi	YH, high(RTC_Data);   zmiennej tablicowej RTC_Data
	ld	R16, Y		; odczyt bajtu 0 tablicy RTC_Data
	ldd	R17, Y+2		; odczyt bajtu 2 tablicy RTC_Data
	cbr	R16, 1<<7		; zerowanie bitu odpowiedzialnego za
					;   odblokowanie zegara RTC
	cbr	R17, 1<<6		; zerowanie bitu odpowiedzialnego za
					;   format przedstawiania czasu (24h)
	st	Y, R16		; modyfikacja bajtu 0 tablicy RTC_Data
	std	Y+2, R17		; modyfikacja bajtu 2 tablicy RTC_Data
	rcall	Zapisz_RTC		; aktualizacja zawartoci pamici ukadu RTC
					;   (powyszy fragment programu ma znaczenie
					;   tylko po utracie zawartoci pamici RTC)

	ldi	ZH, high(T_szablon1<<1)
	ldi	ZL, low(T_szablon1<<1)
	rcall	Pokaz_ciag_LCD	; wywietlanie szablonu w pierwszym wierszu LCD

	ldi	R16, 0x40|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przechodzenie do drugiego wiersza LCD

	ldi	ZH, high(T_szablon2<<1)
	ldi	ZL, low(T_szablon2<<1)
	rcall	Pokaz_ciag_LCD	; wywietlanie szablonu w drugim wierszu LCD


;****************************************************************
Petla:				; ptla gwna programu
	rcall	Czytaj_RTC		; odczyt zawartoci pamici RTC i odwieanie
					;   zmiennej tablicowej RTC_Data
	rcall	Wyswietl_RTC	; odwieanie zawartoci wywietlacza

	ldi	R16, 250
	ldi	R17, 1
	rcall	Czekaj_ms		; opnienie 250 ms

	sbis	I_KLAW, KL_B	; jeli nacinito klawisz B, to...
	rcall	Nastaw_RTC		;   przejd do procedury nastawy zegara

	rjmp	Petla			; zaptl program


;****************************************************************
Czytaj_RTC:				; PODPROGRAM PRZEPISUJCY ZAWARTO PAMICI
					; UKADU RTC DO ZMIENNEJ TABLICOWEJ RTC_Data
	rcall	Odblokuj_I2C	; odblokowanie magistrali I2C
	rcall	I2C_start		; nadawanie sygnau START

	ldi	R16, RTC_ADR|SLA_W
	rcall	I2C_wyslij		; nadawanie adresu RTC i bitu zapisu (SLA+W)

	brts	Czytaj_blad		; jeli nie otrzymano ACK (T=1) lub
	brcs	Czytaj_blad		; wystpi bd transmisji (C=1), to
					;   wykonaj program obsugi bdu
	clr	R16
	rcall	I2C_wyslij		; nadawanie wartoci zerowej (okrelanie
					;   adresu komrki, ktra bdzie odczytywana)
	rcall	I2C_start		; nadawanie sygnau RSTART

	ldi	R16, RTC_ADR|SLA_R
	rcall	I2C_wyslij		; nadawanie adresu RTC i bitu odczytu (SLA+R)

	ldi	XL, low(RTC_Data)	; adowanie wskanika X adresem zmiennej
	ldi	XH, high(RTC_Data);   tablicowej RTC_Data
	ldi	R17, 6		; adowanie licznika ptli
Czytaj_RTC_ptl:
	clt				;   zerowanie znacznika T - dane bd
					;     odbierane z potwierdzeniem (ACK)
	rcall	I2C_odbierz		;   odbir jednego bajtu danych z RTC
	st	X+, R16		;   zapisz odebran warto na kolejnej
					;     pozycji tablicy RTC_Data
	dec	R17
	brne	Czytaj_RTC_ptl	; zaptlenie iteracyjne (ptla x6)

	set				; ustawianie znacznika T - dane bd
					;   odbierane bez potwierdzenia (NACK)	
	rcall	I2C_odbierz		; odbir ostatniego bajtu danych z RTC
	st	X, R16		; zapisz odebran warto na ostatniej
					;   pozycji tablicy RTC_Data

	rcall	I2C_stop		; nadawanie sygnau STOP
	rcall	Zablokuj_I2C	; blokowanie magistrali I2C

	ret

Czytaj_blad:			; w przypadku bdu...
	rjmp	Blad_I2C		;   wywietl informacj o bdzie i
					;     zawie wykonywanie programu

;****************************************************************
Zapisz_RTC:				; PODPROGRAM PRZEPISUJCY ZAWARTO ZMIENNEJ
					; TABLICOWEJ RTC_Data DO UKADU RTC
	rcall	Odblokuj_I2C	; odblokowanie magistrali I2C
	rcall	I2C_start		; nadawanie sygnau START

	ldi	R16, RTC_ADR|SLA_W
	rcall	I2C_wyslij		; nadawanie adresu RTC i bitu zapisu (SLA+W)

	brts	Zapisz_blad		; jeli nie otrzymano ACK (T=1) lub
	brcs	Zapisz_blad		; wystpi bd transmisji (C=1), to
					;   wykonaj program obsugi bdu
	clr	R16
	rcall	I2C_wyslij		; nadawanie wartoci zerowej (okrelanie
					;   adresu komrki, ktra bdzie zapisywana)

	ldi	XL, low(RTC_Data)	; adowanie wskanika X adresem zmiennej
	ldi	XH, high(RTC_Data);   tablicowej RTC_Data
	ldi	R17, 7		; adowanie licznika ptli

Zapisz_RTC_ptl:
	ld	R16, X+		;   odczytaj kolejn komrk tablicy RTC_Data
	rcall	I2C_wyslij		;   i przelij j do ukadu RTC

	brts	Zapisz_blad		;   jeli nie otrzymano ACK (T=1) lub
	brcs	Zapisz_blad		;   wystpi bd transmisji (C=1), to
					;     wykonaj program obsugi bdu
	dec	R17
	brne	Zapisz_RTC_ptl	; zaptlenie iteracyjne (ptla x7)

	rcall	I2C_stop		; nadawanie sygnau STOP
	rcall	Zablokuj_I2C	; blokowanie magistrali I2C

	ret

Zapisz_blad:			; w przypadku bdu...
	rjmp	Blad_I2C		;   wywietl informacj o bdzie i
					;     zawie wykonywanie programu

;****************************************************************
Blad_I2C:				; PODPROGRAM OBSUGI BDU I2C
	rcall	Zablokuj_I2C	;   zablokuj magistral I2C
	rcall	Czysc_LCD		;   wyczy wywietlacz
	ldi	ZH, high(T_blad<<1)
	ldi	ZL, low(T_blad<<1)
	rcall	Pokaz_ciag_LCD	;   wywietl informacj o bdzie
Stoj:	rjmp	Stoj			;   i zatrzymaj dziaanie programu

;****************************************************************
Zablokuj_I2C:			; PODPROGRAM BLOKUJCY MAGISTRAL I2C
	sbi	O_I2C_OFF, I2C_OFF; ustaw stan wysoki na linii blokujcej

	clr	R16
	out	USICR, R16		; wycz modu USI

	ret

;****************************************************************
Odblokuj_I2C:			; PODPROGRAM ODBLOKOWUJCY MAGISTRAL I2C
	ldi	R16, 0xFF
	out	USIDR, R16		; ustaw bity w rejestrze przesuwajcym

	ldi	R16, (1<<USIOIF)|(1<<USIDC)
	out	USISR, R16		; zeruj ewentualne dania przerwa USI

	ldi	R16, (1<<USIWM1)|(1<<USICS1)
	out	USICR, R16		; przywr dziaanie moduu USI

	sbi	O_USI, USI_SDA	; zwolnij lini USI_SDA
	sbi	O_USI, USI_SCL	; zwolnij lini USI_SCL

	cbi	O_I2C_OFF, I2C_OFF; ustaw stan niski na linii blokujcej

	ret

;****************************************************************
.MACRO Wyswietl_cyfre		; MAKRO WYKORZYSTYWANE W PROC. WYSWIETL_RTC
					;   argumenty: A, B, C, D
					;   - A: wiersz wywietlacza (0/1)
					;   - B: kolumna wywietlacza (0...15)
					;   - C: maska wywietlanej cyfry
					;   - D: wybr tetrady (L/H)
	ldi	R17, (@0*0x40+@1)|(1<<LCD_DDADR)
	ldi	R18, @2
	rcall	Wyswietl_BCD_@3
.ENDM

;****************************************************************
Wyswietl_RTC:			; PODPROGRAM ODWIEAJCY DAT I GODZIN
	push	R17			; NA WYWIETLACZU LCD
	push	R18

	ldi	XL, low(RTC_Data)	; adowanie wskanika X adresem zmiennej
	ldi	XH, high(RTC_Data);   tablicowej RTC_Data

	Wyswietl_cyfre 0,15,0x0F,L	; wywietlanie jednoci sekund
	Wyswietl_cyfre 0,14,0x07,H	; wywietlanie dziesitek sekund
	adiw	XL, 1			; inkrementacja wskanika zmiennej RTC_Data
	Wyswietl_cyfre 0,12,0x0F,L	; wywietlanie jednoci minut
	Wyswietl_cyfre 0,11,0x07,H	; wywietlanie dziesitek minut
	adiw	XL, 1			; inkrementacja wskanika zmiennej RTC_Data
	Wyswietl_cyfre 0, 9,0x0F,L	; wywietlanie jednoci godzin
	Wyswietl_cyfre 0, 8,0x03,H	; wywietlanie dziesitek godzin
	adiw	XL, 1			; inkrementacja wskanika zmiennej RTC_Data
	Wyswietl_cyfre 0, 4,0x07,L	; wywietlanie dnia tygodnia
	adiw	XL, 1			; inkrementacja wskanika zmiennej RTC_Data
	Wyswietl_cyfre 1, 4,0x03,H	; wywietlanie dziesitek dni
	Wyswietl_cyfre 1, 5,0x0F,L	; wywietlanie jednoci dni
	adiw	XL, 1			; inkrementacja wskanika zmiennej RTC_Data
	Wyswietl_cyfre 1, 7,0x01,H	; wywietlanie dziesitek miesicy
	Wyswietl_cyfre 1, 8,0x0F,L	; wywietlanie jednoci miesicy
	adiw	XL, 1			; inkrementacja wskanika zmiennej RTC_Data
	Wyswietl_cyfre 1,10,0x0F,H	; wywietlanie dziesitek lat
	Wyswietl_cyfre 1,11,0x0F,L	; wywietlanie jednoci lat

	ldi	R16, 0x11|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przesuwanie kursora poza obszar wywietlania

	pop	R18
	pop	R17
	ret

;****************************************************************
Wyswietl_BCD_L:			; PODPROGRAM WYWIETLAJCY MODSZ TETRAD
					; LICZBY BCD SPOD ADRESU WSKAZYWANEGO PRZEZ X
					;   argumenty: R17, R18
					;   - R17: instrukcja przesuwu kursora LCD
					;   - R18: maska wywietlanej cyfry
	mov	R16, R17
	rcall	Instrukcja_LCD	; przesuwanie kursora w odpowiednie miejsce

	ld	R16, X		; odczytaj liczb BCD spod adresu wskazywanego
					;   przez rejestr indeksowy X
	and	R16, R18		; zamaskuj odczytan warto
	ori	R16, 0b00110000	; zamie cyfr na kod ASCII (+48)
	rcall	Zapisz_LCD		; wywietlanie cyfry na LCD

	ret

;****************************************************************
Wyswietl_BCD_H:			; PODPROGRAM WYWIETLAJCY STARSZ TETRAD
					; LICZBY BCD SPOD ADRESU WSKAZYWANEGO PRZEZ X
					;   argumenty: R17, R18
					;   - R17: instrukcja przesuwu kursora LCD
					;   - R18: maska wywietlanej cyfry
	mov	R16, R17
	rcall	Instrukcja_LCD	; przesuwanie kursora w odpowiednie miejsce

	ld	R16, X		; odczytaj liczb BCD spod adresu wskazywanego
					;   przez rejestr indeksowy X
	swap	R16			; zamie tetrady odczytanej liczby BCD
	and	R16, R18		; zamaskuj otrzyman warto
	ori	R16, 0b00110000	; zamie cyfr na kod ASCII (+48)
	rcall	Zapisz_LCD		; wywietlanie cyfry na LCD

	ret

;****************************************************************
.MACRO Modyfikuj_cyfre		; MAKRO WYKORZYSTYWANE W PROC. NASTAW_RTC
					;   argumenty: A, B, C, D, E
					;   - A: wiersz wywietlacza (0/1)
					;   - B: kolumna wywietlacza (0...15)
					;   - C: maska wywietlanej cyfry
					;   - D: maksymalna warto (1...9)
					;   - E: wybr tetrady (L/H)
	ldi	R17, (@0*0x40+@1)|(1<<LCD_DDADR)
	ldi	R18, @2
	ldi	R19, @3
	rcall	Modyfikuj_BCD_@4
.ENDM

;****************************************************************
Nastaw_RTC:				; PODPROGRAM NASTAWIANIA DATY I GODZINY
	ldi	YL, low(RTC_Data)	; adowanie wskanika Y adresem zmiennej
	ldi	YH, high(RTC_Data);   tablicowej RTC_Data

	ldi	R16, (1<<LCD_STER)|(1<<LCD_SW_ON)|(1<<LCD_SK_ON)|(1<<LCD_SM_OFF)
	rcall	Instrukcja_LCD	; wczanie kursora LCD

	Modyfikuj_cyfre 0,15,0x0F,9,L	; modyfikowanie jednoci sekund
	Modyfikuj_cyfre 0,14,0x07,5,H	; modyfikowanie dziesitek sekund
	adiw	YL, 1			; inkrementacja wskanika zmiennej RTC_Data
	Modyfikuj_cyfre 0,12,0x0F,9,L	; modyfikowanie jednoci minut
	Modyfikuj_cyfre 0,11,0x07,5,H	; modyfikowanie dziesitek minut
	adiw	YL, 1			; inkrementacja wskanika zmiennej RTC_Data
	Modyfikuj_cyfre 0, 9,0x0F,9,L	; modyfikowanie jednoci godzin
	Modyfikuj_cyfre 0, 8,0x03,2,H	; modyfikowanie dziesitek godzin
	adiw	YL, 1			; inkrementacja wskanika zmiennej RTC_Data
	Modyfikuj_cyfre 0, 4,0x07,7,L	; modyfikowanie dnia tygodnia
	adiw	YL, 1			; inkrementacja wskanika zmiennej RTC_Data
	Modyfikuj_cyfre 1, 4,0x03,3,H	; modyfikowanie dziesitek dni
	Modyfikuj_cyfre 1, 5,0x0F,9,L	; modyfikowanie jednoci dni
	adiw	YL, 1			; inkrementacja wskanika zmiennej RTC_Data
	Modyfikuj_cyfre 1, 7,0x01,1,H	; modyfikowanie dziesitek miesicy
	Modyfikuj_cyfre 1, 8,0x0F,9,L	; modyfikowanie jednoci miesicy
	adiw	YL, 1			; inkrementacja wskanika zmiennej RTC_Data
	Modyfikuj_cyfre 1,10,0x0F,9,H	; modyfikowanie dziesitek lat
	Modyfikuj_cyfre 1,11,0x0F,9,L	; modyfikowanie jednoci lat

	ldi	R16, (1<<LCD_STER)|(1<<LCD_SW_ON)|(1<<LCD_SK_OFF)|(1<<LCD_SM_OFF)
	rcall	Instrukcja_LCD	; wyczanie kursora LCD

	rcall	Zapisz_RTC		; przepisywanie nastawionej daty i
					;   godziny do ukadu RTC
	ret

;****************************************************************
Modyfikuj_BCD_L:			; PODPROGRAM MODYFIKUJCY MODSZ TETRAD
					; LICZBY BCD SPOD ADRESU WSKAZYWANEGO PRZEZ Y
					;   argumenty: R17, R18, R19
					;   - R17: instrukcja przesuwu kursora LCD
					;   - R18: maska modyfikowanej cyfry
					;   - R19: maksymalna warto cyfry
	sbis	I_KLAW, KL_B
	rjmp	Modyfikuj_BCD_L	; czekaj na puszczenie klawisza B

	inc	R19			; inkrementacja wartoci maksymalnej cyfry

	mov	R16, R17
	rcall	Instrukcja_LCD	; przesuwanie kursora w odpowiednie miejsce

Modyfikuj_BCD_L0:
	sbis	I_KLAW, KL_A	; jeli nacinito klawisz A, to...
	rjmp	Modyfikuj_BCD_L1	;   przejd do programu zwikszania cyfry
	sbic	I_KLAW, KL_B	; jeli nacinito klawisz B, to...
	rjmp	Modyfikuj_BCD_L0
	rjmp	Modyfikuj_BCD_Lk	;   zakocz dziaanie procedury

Modyfikuj_BCD_L1:
	ld	R20, Y		; odczytaj liczb BCD do modyfikacji
	mov	R21, R20
	and	R21, R18		; zamaskuj odczytan warto
	inc	R21			; inkrementuj j
	cp	R21, R19		; porwnaj z wartoci maksymaln 
	brlo	Modyfikuj_BCD_L2	; jeli przekroczono zakres, to...
	clr	R21			;   wyzeruj modyfikowan cyfr
Modyfikuj_BCD_L2:
	com	R18
	and	R20, R18
	com	R18
	or	R20, R21		; podmie modyfikowan cz liczby BCD
	st	Y, R20		; i zapisz j ostatecznie w pamici

	rcall	Wyswietl_RTC	; odwieanie wywietlacza

	mov	R16, R17
	rcall	Instrukcja_LCD	; przesuwanie kursora w odpowiednie miejsce

	push	R17
	ldi	R16, 200
	ldi	R17, 1
	rcall	Czekaj_ms		; opnienie o 200 ms (czas repetycji)
	pop	R17

	rjmp	Modyfikuj_BCD_L0	; zaptlenie programu modyfikujcego

Modyfikuj_BCD_Lk:
	ret

;****************************************************************
Modyfikuj_BCD_H:			; PODPROGRAM MODYFIKUJCY STARSZ TETRAD
					; LICZBY BCD SPOD ADRESU WSKAZYWANEGO PRZEZ Y
					;   argumenty: R17, R18, R19
					;   - R17: instrukcja przesuwu kursora LCD
					;   - R18: maska modyfikowanej cyfry
					;   - R19: maksymalna warto cyfry
	sbis	I_KLAW, KL_B
	rjmp	Modyfikuj_BCD_H	; czekaj na puszczenie klawisza B

	inc	R19			; inkrementacja wartoci maksymalnej cyfry

	mov	R16, R17
	rcall	Instrukcja_LCD	; przesuwanie kursora w odpowiednie miejsce

Modyfikuj_BCD_H0:
	sbis	I_KLAW, KL_A	; jeli nacinito klawisz A, to...
	rjmp	Modyfikuj_BCD_H1	;   przejd do programu zwikszania cyfry
	sbic	I_KLAW, KL_B	; jeli nacinito klawisz B, to...
	rjmp	Modyfikuj_BCD_H0
	rjmp	Modyfikuj_BCD_Hk	;   zakocz dziaanie procedury

Modyfikuj_BCD_H1:
	ld	R20, Y		; odczytaj liczb BCD do modyfikacji
	swap	R20			; zamie tetrady odczytanej wartoci
	mov	R21, R20
	and	R21, R18		; zamaskuj j
	inc	R21			; inkrementuj jej warto
	cp	R21, R19		; porwnaj z wartoci maksymaln 
	brlo	Modyfikuj_BCD_H2	; jeli przekroczono zakres, to...
	clr	R21			;   wyzeruj modyfikowan cyfr
Modyfikuj_BCD_H2:
	com	R18
	and	R20, R18
	com	R18
	or	R20, R21		; podmie modyfikowan cz liczby BCD
	swap	R20			; przywr jej oryginaln posta
	st	Y, R20		; i zapisz j ostatecznie w pamici

	rcall	Wyswietl_RTC	; odwieanie wywietlacza

	mov	R16, R17
	rcall	Instrukcja_LCD	; przesuwanie kursora w odpowiednie miejsce

	push	R17
	ldi	R16, 200
	ldi	R17, 1
	rcall	Czekaj_ms		; opnienie o 200 ms (czas repetycji)
	pop	R17

	rjmp	Modyfikuj_BCD_H0	; zaptlenie programu modyfikujcego

Modyfikuj_BCD_Hk:
	ret

;****************************************************************
					; DOCZANE PODPROGRAMY
.INCLUDE "czekaj_ms.inc"	;   oczekiwanie milisekundowe
.INCLUDE "czekaj_us.inc"	;   oczekiwanie mikrosekundowe
.INCLUDE "usi_i2c_m.inc"	;   obsuga moduu USI w nadrzdnym trybie I2C
.INCLUDE "lcd.inc"		;   obsuga wywietlacza LCD
