; PODPROGRAMY OBSUGI INTERFEJSU I2C W TRYBIE NADRZDNYM POPRZEZ USI
; Rafa Baranowski (C) 2005
;
; Etykiety procedur:
; 	Ini_USI_I2C		- inicjalizacja moduu USI w nadrzdnym trybie I2C
;	I2C_start		- generacja sygnau START
;	I2C_stop		- generacja sygnau STOP
;	I2C_wyslij		- zapis wartoci 8-bitowej
;	I2C_odbierz		- odczyt wartoci 8-bitowej
;	I2C_wait		- opnienie > 5 us
;
; Wymagane stae:
;	SYS_FREQ 		- czstotliwo pracy w MHz
;	K_USI			- rejestr kierunku portu USI
;	O_USI			- rejestr wyjciowy portu USI
;	I_USI			- rejestr wejciowy portu USI
;	USI_SDA		- nr linii dla sygnau DI/SDA
;	USI_SCL		- nr linii dla sygnau USCK/SCL
;

;****************************************************************
;	PROCEDURA INICJALIZUJCA MODU USI W NADRZDNYM TRYBIE I2C
;
;
					; deklaracje etykiet adresowych
.EQU	SLA_R	= 1			;   bit R przy adresowaniu SLA+R
.EQU	SLA_W	= 0			;   bit W przy adresowaniu SLA+W
.EQU	GCA	= 0			;   adres zbiorowy

Ini_USI_I2C:
	push	R16

	ldi	R16, 0xFF
	out	USIDR, R16		; ustaw bity w rejestrze przesuwajcym

	ldi	R16, (1<<USIOIF)|(1<<USIDC)
	out	USISR, R16		; anuluj ewentualne dania przerwa

	ldi	R16, (1<<USIWM1)|(1<<USICS1)
	out	USICR, R16		; wcz modu USI w nadrzdnym trybie I2C

	sbi	O_USI, USI_SDA	; zwolnij lini SDA
	sbi	O_USI, USI_SCL	; zwolnij lini SCL
	sbi	K_USI, USI_SDA	; wcz tryb wyjciowy na linii SDA
	sbi	K_USI, USI_SCL	; wcz tryb wyjciowy na linii SCL

	pop	R16
	ret

;****************************************************************
;	PROCEDURA GENERUJCA SYGNA START LUB RSTART
;
I2C_start:
	cbi	O_USI, USI_SCL	; wymu stan niski na linii SCL

	ldi	R16, (1<<USIOIF)|(1<<USIDC)|(1<<USISIF)|(1<<USIPF)
	out	USISR, R16		; zeruj znaczniki przerwa i licznik USI

	ldi	R16, 0xFF
	out	USIDR, R16		; ustaw bity w rejestrze przesuwajcym

	sbi	O_USI, USI_SDA	; zwolnij lini SDA
I2C_start_1:
	sbis	I_USI, USI_SDA	; oczekuj na faktyczne zwolnienie SDA
	rjmp	I2C_start_1

	sbi	O_USI, USI_SCL	; zwolnij lini SCL
I2C_start_2:
	sbis	I_USI, USI_SCL	; oczekuj na faktyczne zwolnienie SCL
	rjmp	I2C_start_2

	rcall	I2C_wait		; wprowad opnienie
	cbi	O_USI, USI_SDA	; wymu stan niski na linii SDA (START)
	rcall	I2C_wait		; wprowad opnienie
	cbi	O_USI, USI_SCL	; wymu stan niski na linii SCL

	ldi	R16, (1<<USIOIF)|(1<<USIDC)|(1<<USISIF)|(1<<USIPF)
	out	USISR, R16		; zeruj znaczniki przerwa i licznik USI

	ret


;****************************************************************
;		PROCEDURA GENERUJCA SYGNA STOP
;
I2C_stop:
	cbi	O_USI, USI_SCL	; wymu stan niski na linii SCL
	cbi	O_USI, USI_SDA	; wymu stan niski na linii SDA

	ldi	R16, (1<<USIOIF)|(1<<USIDC)|(1<<USISIF)|(1<<USIPF)
	out	USISR, R16		; zeruj znaczniki przerwa i licznik USI

	sbi	O_USI, USI_SCL	; zwolnij lini SCL
I2C_stop_1:
	sbis	I_USI, USI_SCL	; oczekuj na faktyczne zwolnienie SCL
	rjmp	I2C_stop_1

	rcall	I2C_wait		; wprowad opnienie
	ldi	R16, 0xFF
	out	USIDR, R16		; ustaw bity w rejestrze przesuwajcym
	sbi	O_USI, USI_SDA	; zwolnij lini SDA (STOP)
	rcall	I2C_wait		; wprowad opnienie

	ldi	R16, (1<<USIOIF)|(1<<USIDC)|(1<<USISIF)|(1<<USIPF)
	out	USISR, R16		; zeruj znaczniki przerwa i licznik USI

	ret


;****************************************************************
;		PROCEDURA PRZESYAJCA WARTO 8-BITOW
;
; Parametry:
;	R16		- warto do wysania
;
; Wartoci wyjciowe:
;	T (SREG)	- odebrane potwierdzenie (T=1 => NACK)
;	C (SREG)	- znacznik bdu (C=1 => bd)
;
I2C_wyslij:
	out	USIDR, R16		; zaaduj rejestr przesuwajcy
					;   wartoci przesyan z rejestru R16
	sbi	O_USI, USI_SDA	; zwolnij lini SDA

	ldi	R16, (1<<USIOIF)|(1<<USIDC)|(1<<USISIF)|(1<<USIPF)
	out	USISR, R16		; zeruj znaczniki przerwa i licznik USI

I2C_wyslij_ptl:			; ptla przesyajca bajt
	rcall	I2C_wait		;   wprowad opnienie

	sbi	O_USI, USI_SCL	;   zwolnij lini SCL (dane wane)
I2C_wyslij_1:
	sbis	I_USI, USI_SCL	;   oczekuj na faktyczne zwolnienie SCL
	rjmp	I2C_wyslij_1

	rcall	I2C_wait		;   wprowad opnienie

	cbi	O_USI, USI_SCL	;   wymu stan niski na SCL (dane niewane)

	in	R16, USISR
	andi	R16, (1<<USIDC)|(1<<USISIF)|(1<<USIPF)
	brne	I2C_wyslij_blad	;   jeli wystpio zdarzenie kolizji,
					;     sygna START lub STOP, to wykonaj
					;     program obsugi bdu
	in	R16, USISR
	sbrs	R16, USIOIF		;   jeli nie nastpio przepenienie USI, to
	rjmp	I2C_wyslij_ptl	;     przejd do przesyania kolejnego bitu

	ldi	R16, 0xFF
	out	USIDR, R16		; ustaw bity w rejestrze przesuwajcym

	rcall	I2C_wait		; wprowad opnienie

	ldi	R16, (1<<USIOIF)
	out	USISR, R16		; zeruj znacznik przepenienia USI

	sbi	O_USI, USI_SCL	; zwolnij lini SCL
I2C_wyslij_2:
	sbis	I_USI, USI_SCL	; oczekuj na faktyczne zwolnienie SCL
	rjmp	I2C_wyslij_2

	rcall	I2C_wait		; wprowad opnienie
	in	R16, I_USI
	bst	R16, USI_SDA	; odczytaj bit potwierdzenia (ACK/NACK)
					;   i zapisz go w znaczniku T (SREG)

	cbi	O_USI, USI_SCL	; wymu stan niski na linii SCL

	in	R16, USISR
	andi	R16, (1<<USISIF)|(1<<USIPF)
	brne	I2C_wyslij_blad	; jeli wykryto sygna START lub STOP,
					;   to wykonaj program obsugi bdu
	
	clc				; zeruj umowny znacznik bdu
	ret				; i zakocz dziaanie procedury

I2C_wyslij_blad:			; w przypadku bdu...
	sec				;   ustaw umowny znacznik bdu
	ret				;   i zakocz dziaanie procedury


;****************************************************************
;		PROCEDURA ODBIERAJCA WARTO 8-BITOW
;
; Parametry:
;	T (SREG)	- przesyane potwierdzenie (T=1 => NACK)
;
; Wartoci wyjciowe:
;	R16		- warto odebrana
;	C (SREG)	- znacznik bdu (C=1 => bd)
;
I2C_odbierz:
	push	R17
	
	ldi	R17, 0xFF
	out	USIDR, R17		; ustaw bity w rejestrze przesuwajcym

	sbi	O_USI, USI_SDA	; zwolnij lini SDA

	ldi	R17, (1<<USIOIF)|(1<<USIDC)|(1<<USISIF)|(1<<USIPF)
	out	USISR, R17		; zeruj znaczniki przerwa i licznik USI

I2C_odbierz_ptl:			; ptla odbierajca bajt
	rcall	I2C_wait		; wprowad opnienie

	sbi	O_USI, USI_SCL	; zwolnij lini SCL (dane wane)
I2C_odbierz_1:
	sbis	I_USI, USI_SCL	; oczekuj na faktyczne zwolnienie SCL
	rjmp	I2C_odbierz_1

	rcall	I2C_wait		; wprowad opnienie

	cbi	O_USI, USI_SCL	; wymu stan niski na SCL (dane niewane)

	in	R17, USISR
	andi	R17, (1<<USISIF)|(1<<USIPF)
	brne	I2C_odbierz_blad	;   jeli wykryto sygna START lub STOP,
					;     to wykonaj program obsugi bdu

	in	R17, USISR
	sbrs	R17, USIOIF		;   jeli nie nastpio przepenienie USI, to
	rjmp	I2C_odbierz_ptl	;     przejd do odbierania kolejnego bitu

	in	R16, USIDR		; zapisz warto odebran w rejestrze R16

	bld	R17, 7
	out	USIDR, R17		; zaaduj najstarszy bit rejestru
					;   przesuwajcego (lini SDA) wartoci
					;   znacznika T ze SREG (bitem ACK/NACK)

	rcall	I2C_wait		; wprowad opnienie

	ldi	R17, (1<<USIOIF)
	out	USISR, R17		; zeruj znacznik przepenienia USI

	sbi	O_USI, USI_SCL	; zwolnij lini SCL
I2C_odbierz_2:
	sbis	I_USI, USI_SCL	; oczekuj na faktyczne zwolnienie SCL
	rjmp	I2C_odbierz_2

	rcall	I2C_wait		; wprowad opnienie
					; (nadawanie bitu potwierdzenia ACK/NACK)

	cbi	O_USI, USI_SCL	; wymu stan niski na linii SCL

	in	R17, USISR
	andi	R17, (1<<USIDC)|(1<<USISIF)|(1<<USIPF)
	brne	I2C_odbierz_blad	; jeli wystpio zdarzenie kolizji,
					;   sygna START lub STOP, to wykonaj
					;   program obsugi bdu

	clc				; zeruj umowny znacznik bdu
	pop	R17
	ret				; i zakocz dziaanie procedury

I2C_odbierz_blad:			; w przypadku bdu...
	sec				;   ustaw umowny znacznik bdu
	pop	R17
	ret				;   i zakocz dziaanie procedury



;****************************************************************
;		PROCEDURA WPROWADZAJCA OPӏNIENIE > 5 us
;
I2C_wait:
.IF SYS_FREQ == 1			; jeli cz. systemowa = 1 MHz:
	ret				;   powr (czas wykonywania 7 cykli => 7us)
.ELIF SYS_FREQ == 2		; jeli cz. systemowa = 2 MHz:
	push	R16
	pop	R16			;   wykonaj operacj bez efektu
	ret				;   (czas wykonywania 11 cykli => 5,5 us)
.ELIF	SYS_FREQ >= 3		; jeli cz. systemowa >= 3 MHz:
	push	R16
	ldi	R16, SYS_FREQ-2
I2C_wait_1:				;   wykonuj ptl opniajc
	nop
	nop
	dec	R16
	brne	I2C_wait_1

	pop	R16			;   (czas wykonywania 2+5*SYS_FREQ cykli
	ret				;    => 5 us + 2/SYS_FREQ us)
.ENDIF
